﻿using System;
using System.Drawing;
using System.IO;
using System.Text;

using System.Windows.Forms;

namespace nVault
{
    public partial class frmnVault : Form
    {
        public frmnVault()
        {
            InitializeComponent();
            dataGridView1.ColumnHeadersDefaultCellStyle.Font = new Font("Calibri", 10);
            dataGridView1.DefaultCellStyle.Font = new Font("Calibri", 10);
        }

        private void LoadVault()
        {
            Int32 VM = 0x6E564C54;

            if (!File.Exists(textBox1.Text))
                return;

            FileStream fs = File.OpenRead(textBox1.Text);

            var iFile = new BinaryReader(fs);

            Int32 iVaultMagic = iFile.ReadInt32();
            byte iVerMinor = iFile.ReadByte();
            byte iVerMajor = iFile.ReadByte();
            statusLabelMagic.Text = "nVault Magic: " + iVaultMagic;
            statusLabelVersion.Text = "nVault Version: " + iVerMajor + "." + iVerMinor;

            dataGridView1.Rows.Clear();

            if ((VM == iVaultMagic) && (iVerMinor == 0) && (iVerMajor == 2))
            {
                Int32 iEntries = iFile.ReadInt32();
                statusLabelEntries.Text = "Records: " + iEntries;

                Int32 iEntryCounter = 0;

                while (iEntryCounter < iEntries)
                {
                    Int32 iTimeStamp = iFile.ReadInt32();
                    byte bKeyLen = iFile.ReadByte();
                    short sValLen = iFile.ReadInt16();

                    byte[] bKey;
                    byte[] bVal;

                    bKey = iFile.ReadBytes(bKeyLen);
                    bVal = iFile.ReadBytes(sValLen);

                    DateTime dateTime = new DateTime(1970, 1, 1, 0, 0, 0, 0, DateTimeKind.Local);
                    dateTime = dateTime.AddSeconds(iTimeStamp).ToLocalTime();

                    string[] RowData = { System.Text.Encoding.Default.GetString(bKey), System.Text.Encoding.Default.GetString(bVal), iTimeStamp == 0 ? "Permanent" : "" + dateTime, "" + iTimeStamp };
                    dataGridView1.Rows.Add(RowData);

                    iEntryCounter++;
                }
            }
            else
            {
                Console.WriteLine("Invalid nVault File");
            }
            iFile.Close();
        }

        private void SaveVault()
        {
            FileStream fs = File.OpenWrite(textBox1.Text);
            var iFile = new BinaryWriter(fs);

            int i;
            uint iVM = 0x6E564C54;
            byte bMinorVer = 0;
            byte bMajorVer = 2;
            uint iEntries = (uint)dataGridView1.Rows.Count - 1;

            uint iTimeStamp;
            byte bKeyLen;
            ushort sValLen;

            iFile.Write(iVM);
            iFile.Write(bMinorVer);
            iFile.Write(bMajorVer);
            iFile.Write(iEntries);

            for (i = 0; i < (dataGridView1.Rows.Count - 1); i++)
            {
                //Write timestamp
                iTimeStamp = (uint)Convert.ToUInt32(dataGridView1.Rows[i].Cells[3].Value.ToString());
                iFile.Write(iTimeStamp);

                //Write key len
                String keyVal = (dataGridView1.Rows[i].Cells[0].Value == null) ? "" : dataGridView1.Rows[i].Cells[0].Value.ToString();
                bKeyLen = (byte)keyVal.Length;
                iFile.Write(bKeyLen);

                //Write val len
                String valVal = (dataGridView1.Rows[i].Cells[1].Value == null) ? "" : dataGridView1.Rows[i].Cells[1].Value.ToString();
                sValLen = (ushort)valVal.Length;
                iFile.Write(sValLen);

                byte[] szKey;
                byte[] szVal;

                //Write key and val
                if (dataGridView1.Rows[i].Cells[0].Value != null)
                {
                    szKey = Encoding.ASCII.GetBytes(dataGridView1.Rows[i].Cells[0].Value.ToString());
                }
                else
                {
                    szKey = new byte[1];
                }

                if (dataGridView1.Rows[i].Cells[1].Value != null)
                {
                    szVal = Encoding.ASCII.GetBytes(dataGridView1.Rows[i].Cells[1].Value.ToString());
                }
                else
                {
                    szVal = new byte[1];
                }

                iFile.Write(szKey);
                iFile.Write(szVal);
            }
            iFile.Close();
        }
        
        private void BtnNew_Click(object sender, EventArgs e)
        {
            dataGridView1.Rows.Clear();

            statusLabelMagic.Text = "nVault Magic: N/A";
            statusLabelVersion.Text = "nVault Version: N/A";
            statusLabelEntries.Text = "Records: N/A";
        }
        private void BtnOpen_Click(object sender, EventArgs e)
        {
            openFileDialog1.ShowDialog();

            textBox1.Text = openFileDialog1.FileName;
            if (File.Exists(textBox1.Text))
            {
                LoadVault();
            }
        }

        private void BtnSave_Click(object sender, EventArgs e)
        {
            if (dataGridView1.Rows.Count == 1)
            {
                MessageBox.Show("No records to save.", "Error");
                return;
            }

            int i;

            //Validate all entries are made
            MessageBoxButtons buttons = MessageBoxButtons.OK;
            for (i = 0; i < (dataGridView1.Rows.Count - 1); i++)
            {
                if (/*(dataGridView1.Rows[i].Cells[0].Value == null) || 
                    (dataGridView1.Rows[i].Cells[1].Value == null) ||*/ 
                    (dataGridView1.Rows[i].Cells[3].Value == null))
                {
                    MessageBox.Show("All vault records must have a timestamp. Please see row# " + (i+1) + ".", "Error", buttons);
                    return;
                }
            }

            saveFileDialog1.ShowDialog();
            String szFile = saveFileDialog1.FileName;
            if (szFile.Length > 0)
            {
                textBox1.Text = saveFileDialog1.FileName;
                SaveVault();
            }
        }

        private void DataGridView1_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {
            if ((e.ColumnIndex==3) && (dataGridView1.Rows[e.RowIndex].Cells[3].Value != null))
            {
                    try
                    {
                        Int32 iTimeStamp = Convert.ToInt32(dataGridView1.Rows[e.RowIndex].Cells[3].Value);
                        DateTime dateTime = new DateTime(1970, 1, 1, 0, 0, 0, 0, DateTimeKind.Local);
                        dateTime = dateTime.AddSeconds(iTimeStamp).ToLocalTime();

                        dataGridView1.Rows[e.RowIndex].Cells[3].Value = iTimeStamp.ToString();
                        dataGridView1.Rows[e.RowIndex].Cells[2].Value = (iTimeStamp == 0) ? "Permanent" : dateTime.ToString();
                    }
                    catch 
                    {
                        MessageBox.Show("Raw timestamp value must be numeric. See row # " + (e.RowIndex + 1) + ".", "Error");
                    }
            }
        }

        private void DataGridView1_RowsAdded(object sender, DataGridViewRowsAddedEventArgs e)
        {
            statusLabelEntries.Text = "Records: " + (dataGridView1.Rows.Count - 1);
        }

        private void DataGridView1_RowsRemoved(object sender, DataGridViewRowsRemovedEventArgs e)
        {
            statusLabelEntries.Text = "Records: " + (dataGridView1.Rows.Count - 1);
        }

        private void DateTimePicker2_Leave(object sender, EventArgs e)
        {
            Int32 unixTimestamp = (int)dateTimePicker2.Value.Subtract(new DateTime(1970, 1, 1)).TotalSeconds;
            dataGridView1.Rows[dataGridView1.CurrentCell.RowIndex].Cells[2].Value = dateTimePicker2.Value;
            dataGridView1.Rows[dataGridView1.CurrentCell.RowIndex].Cells[3].Value = unixTimestamp.ToString();
            dateTimePicker2.Visible = false;
        }

        private void DataGridView1_EditingControlShowing(object sender, DataGridViewEditingControlShowingEventArgs e)
        {
            if (dataGridView1.CurrentCell.ColumnIndex == 2)
            {
                Rectangle cellRectangle = dataGridView1.GetCellDisplayRectangle(dataGridView1.CurrentCell.ColumnIndex, dataGridView1.CurrentCell.RowIndex, true);

                dateTimePicker2.Left = cellRectangle.Left + 25;
                dateTimePicker2.Top = cellRectangle.Top + 75;
                dateTimePicker2.Height = cellRectangle.Height;
                dateTimePicker2.Width = cellRectangle.Width;

                if (dataGridView1.Rows[dataGridView1.CurrentCell.RowIndex].Cells[2].Value != null)
                    dateTimePicker2.Value = Convert.ToDateTime(dataGridView1.Rows[dataGridView1.CurrentCell.RowIndex].Cells[2].Value);

                dateTimePicker2.Visible = true;
            }
        }
    }
}
